@@ -29,6 +29,10 @@ meta_dir = code_dir / "meta"
 working_dir = code_dir / "working"
 
 
+def get_ip():
+    return subprocess.check_output(["hostname", "-I"]).split()[0].decode()
+
+
 def update_system(package):
     print(
         subprocess.run(
@@ -70,7 +74,6 @@ class System:
         """"""
         if not web.tx.user.is_owner:
             return "owner only"
-        ip_address = subprocess.check_output(["hostname", "-I"]).split()[0].decode()
         try:
             with open("/home/gaea/app/run/onion") as fp:
                 onion = fp.read()
@@ -80,7 +83,7 @@ class System:
         webint_metadata = importlib.metadata.metadata("webint")
         webint_versions = get_versions("webint")
         return app.view.index(
-            ip_address,
+            get_ip(),
             onion,
             domains,
             web.tx.app.cfg,
@@ -100,6 +103,56 @@ class Addresses:
         return "addresses have been updated"
 
 
+@app.control("addresses/domains")
+class Domains:
+    """System addresses."""
+
+    def get(self):
+        """"""
+        form = web.form("domain")
+        records = {}
+        for record in ("A", "CNAME", "MX", "NS"):
+            try:
+                records[record] = web.dns.resolve(form.domain, record)
+            except (web.dns.NoAnswer, web.dns.NXDOMAIN):
+                pass
+        return app.view.addresses.domains(get_ip(), form.domain, records)
+
+    def post(self):
+        form = web.form("domain")
+        web.enqueue(add_domain, form.domain)
+        return "adding domain"
+
+
+def add_domain(domain):
+    ip = get_ip()
+    with open("onion") as fp:
+        onion = fp.read()
+
+    domain_path = pathlib.Path("domains")
+    with domain_path.open() as fp:
+        domains = {domain: True for domain in fp.readlines()}
+    domains[domain] = False
+
+    nginx_conf_path = pathlib.Path("/etc/nginx/nginx.conf")
+    with nginx_conf_path.open("w") as fp:
+        fp.write(web.host.templates.nginx(ip, onion, domains))
+    subprocess.call(["sudo", "service", "nginx", "restart"])
+
+    web.generate_cert(domain)
+    subprocess.call(["cp", f"certs/{domain}/domain.key", "/etc/ssl/private"])
+    subprocess.call(["cp", f"certs/{domain}/domain.crt", "/etc/ssl/certs"])
+
+    domains[domain] = True
+    with nginx_conf_path.open("w") as fp:
+        fp.write(web.host.templates.nginx(ip, onion, domains))
+    subprocess.call(["sudo", "service", "nginx", "restart"])
+
+    with domain_path.open("w") as fp:
+        for domain in domains:
+            print(domain, file=fp)
+
+
 @app.control("settings")
 class Settings:
     """System settings."""
